@@ -11,13 +11,15 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from typing import Union
 
 import torch
 from torch import optim
 
 from pytorch_lightning.accelerators.accelerator import Accelerator
-from pytorch_lightning.distributed import LightningDistributed
+from pytorch_lightning.core.lightning import LightningModule
 from pytorch_lightning.core.step_result import Result
+from pytorch_lightning.distributed import LightningDistributed
 from pytorch_lightning.overrides.data_parallel import LightningDataParallel
 from pytorch_lightning.utilities import AMPType
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
@@ -61,6 +63,8 @@ class DataParallelAccelerator(Accelerator):
         if self.trainer.amp_backend:
             model = self.__init_half_precision(model)
 
+        self.trainer.convert_to_lightning_optimizers()
+
         self.trainer.model = model
 
     def __init_torch_data_parallel(self, model):
@@ -172,3 +176,12 @@ class DataParallelAccelerator(Accelerator):
                 scheduler.__class__.__mro__[idx].__init__(scheduler, optimizer)
                 if state is not None:
                     scheduler.load_state_dict(state)
+
+    def get_reference_model(self, model) -> LightningModule:
+        if isinstance(model, LightningDataParallel):
+            return model.module
+        return model
+
+    @property
+    def require_distributed_sampler(self):
+        return False
