@@ -11,35 +11,30 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from pytorch_lightning.metrics.functional.average_precision import average_precision
 from pytorch_lightning.metrics.functional.classification import (
     accuracy,
     auc,
     auroc,
-    average_precision,
-    confusion_matrix,
     dice_score,
-    multiclass_precision_recall_curve,
-    multiclass_roc,
+    multiclass_auroc,
     precision,
     precision_recall,
-    precision_recall_curve,
     recall,
-    roc,
     stat_scores,
     stat_scores_multiple_classes,
-    to_categorical,
-    to_onehot,
     iou,
 )
-from pytorch_lightning.metrics.functional.nlp import bleu_score
-from pytorch_lightning.metrics.functional.self_supervised import (
-    embedding_similarity
-)
+from pytorch_lightning.metrics.functional.confusion_matrix import confusion_matrix
 # TODO: unify metrics between class and functional, add below
 from pytorch_lightning.metrics.functional.explained_variance import explained_variance
+from pytorch_lightning.metrics.functional.f_beta import fbeta, f1
 from pytorch_lightning.metrics.functional.mean_absolute_error import mean_absolute_error
 from pytorch_lightning.metrics.functional.mean_squared_error import mean_squared_error
 from pytorch_lightning.metrics.functional.mean_squared_log_error import mean_squared_log_error
+from pytorch_lightning.metrics.functional.nlp import bleu_score
+from pytorch_lightning.metrics.functional.precision_recall_curve import precision_recall_curve
 from pytorch_lightning.metrics.functional.psnr import psnr
+from pytorch_lightning.metrics.functional.roc import roc
+from pytorch_lightning.metrics.functional.self_supervised import embedding_similarity
 from pytorch_lightning.metrics.functional.ssim import ssim
-from pytorch_lightning.metrics.functional.f_beta import fbeta, f1
