@@ -11,7 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
+from pytorch_lightning import _logger as log
 from pytorch_lightning.core.lightning import LightningModule
 from pytorch_lightning.utilities import rank_zero_warn
 from pytorch_lightning.utilities.exceptions import MisconfigurationException
@@ -69,6 +69,37 @@ class ConfigValidator(object):
                 ' `training_step()`, `train_dataloader()` and `configure_optimizers()` to be defined.'
             )
 
+        trainer = self.trainer
+
+        trainer.overriden_optimizer_step = is_overridden('optimizer_step', model)
+        trainer.overriden_optimizer_zero_grad = is_overridden('optimizer_zero_grad', model)
+
+        enable_pl_optimizer = trainer._enable_pl_optimizer
+        automatic_optimization = trainer.train_loop.automatic_optimization
+        if trainer.overriden_optimizer_step and not enable_pl_optimizer and automatic_optimization:
+            rank_zero_warn(
+                "When overriding `LightningModule` optimizer_step with"
+                " `Trainer(..., enable_pl_optimizer=False, automatic_optimization=True, ...)`,"
+                " we won't be calling `.zero_grad` we can't assume when you call your `optimizer.step()`."
+                " For Lightning to take care of it, please use `Trainer(enable_pl_optimizer=True)`."
+            )
+
+        going_to_accumulate_grad_batches = trainer.accumulation_scheduler.going_to_accumulate_grad_batches()
+
+        has_overriden_optimization_functions = trainer.overriden_optimizer_step or trainer.overriden_optimizer_zero_grad
+        if (has_overriden_optimization_functions) and going_to_accumulate_grad_batches and automatic_optimization:
+            raise MisconfigurationException(
+                'When overriding `LightningModule` optimizer_step or optimizer_zero_grad with '
+                '`Trainer(automatic_optimization=True, ...)`, `accumulate_grad_batches` should to be 1.'
+                ' It ensures optimizer_step or optimizer_zero_grad are called on every batch.'
+            )
+
+        if (enable_pl_optimizer) and trainer.overriden_optimizer_zero_grad and not automatic_optimization:
+            raise MisconfigurationException(
+                'When overriding `LightningModule` optimizer_zero_grad with  '
+                '`Trainer(automatic_optimization=False, enable_pl_optimizer=True, ...) is not supported'
+            )
+
     def __verify_eval_loop_configuration(self, model, eval_loop_name):
         step_name = f'{eval_loop_name}_step'
 
